@@ -10,6 +10,7 @@ import time
 
 import aiohttp
 from aiohttp import ContentTypeError, FormData
+from pyrate_limiter import BucketFullException, Duration, Limiter, Rate
 
 from roborock.containers import HomeData, HomeDataRoom, HomeDataScene, ProductResponse, RRiot, UserData
 from roborock.exceptions import (
@@ -21,6 +22,7 @@ from roborock.exceptions import (
     RoborockInvalidUserAgreement,
     RoborockMissingParameters,
     RoborockNoUserAgreement,
+    RoborockRateLimit,
     RoborockTooFrequentCodeRequests,
     RoborockTooManyRequest,
     RoborockUrlException,
@@ -30,6 +32,22 @@ _LOGGER = logging.getLogger(__name__)
 
 
 class RoborockApiClient:
+    _LOGIN_RATES = [
+        Rate(1, Duration.SECOND),
+        Rate(3, Duration.MINUTE),
+        Rate(10, Duration.HOUR),
+        Rate(20, Duration.DAY),
+    ]
+    _HOME_DATA_RATES = [
+        Rate(1, Duration.SECOND),
+        Rate(5, Duration.MINUTE),
+        Rate(15, Duration.HOUR),
+        Rate(40, Duration.DAY),
+    ]
+
+    _login_limiter = Limiter(_LOGIN_RATES)
+    _home_data_limiter = Limiter(_HOME_DATA_RATES)
+
     def __init__(self, username: str, base_url=None, session: aiohttp.ClientSession | None = None) -> None:
         """Sample API Client."""
         self._username = username
@@ -50,11 +68,12 @@ class RoborockApiClient:
                 raise RoborockUrlException("get url by email returned None")
             response_code = response.get("code")
             if response_code != 200:
+                _LOGGER.info("Get base url failed for %s with the following context: %s", self._username, response)
                 if response_code == 2003:
                     raise RoborockInvalidEmail("Your email was incorrectly formatted.")
                 elif response_code == 1001:
                     raise RoborockMissingParameters(
-                        "You are missing parameters for this request, are you sure you " "entered your username?"
+                        "You are missing parameters for this request, are you sure you entered your username?"
                     )
                 elif response_code == 9002:
                     raise RoborockTooManyRequest("Please temporarily disable making requests and try again later.")
@@ -71,39 +90,6 @@ class RoborockApiClient:
         md5.update(self._device_identifier.encode())
         return base64.b64encode(md5.digest()).decode()
 
-    def _process_extra_hawk_values(self, values: dict | None) -> str:
-        if values is None:
-            return ""
-        else:
-            sorted_keys = sorted(values.keys())
-            result = []
-            for key in sorted_keys:
-                value = values.get(key)
-                result.append(f"{key}={value}")
-            return hashlib.md5("&".join(result).encode()).hexdigest()
-
-    def _get_hawk_authentication(
-        self, rriot: RRiot, url: str, formdata: dict | None = None, params: dict | None = None
-    ) -> str:
-        timestamp = math.floor(time.time())
-        nonce = secrets.token_urlsafe(6)
-        formdata_str = self._process_extra_hawk_values(formdata)
-        params_str = self._process_extra_hawk_values(params)
-
-        prestr = ":".join(
-            [
-                rriot.u,
-                rriot.s,
-                nonce,
-                str(timestamp),
-                hashlib.md5(url.encode()).hexdigest(),
-                params_str,
-                formdata_str,
-            ]
-        )
-        mac = base64.b64encode(hmac.new(rriot.h.encode(), prestr.encode(), hashlib.sha256).digest()).decode()
-        return f'Hawk id="{rriot.u}",s="{rriot.s}",ts="{timestamp}",nonce="{nonce}",mac="{mac}"'
-
     async def nc_prepare(self, user_data: UserData, timezone: str) -> dict:
         """This gets a few critical parameters for adding a device to your account."""
         if (
@@ -125,7 +111,7 @@ class RoborockApiClient:
             "post",
             "/nc/prepare",
             headers={
-                "Authorization": self._get_hawk_authentication(
+                "Authorization": _get_hawk_authentication(
                     user_data.rriot, "/nc/prepare", {"hid": hid, "tzid": timezone}
                 ),
             },
@@ -158,7 +144,7 @@ class RoborockApiClient:
             "GET",
             "/user/devices/newadd",
             headers={
-                "Authorization": self._get_hawk_authentication(
+                "Authorization": _get_hawk_authentication(
                     user_data.rriot, "/user/devices/newadd", params={"s": s, "t": t}
                 ),
             },
@@ -175,6 +161,11 @@ class RoborockApiClient:
         return add_device_response["result"]
 
     async def request_code(self) -> None:
+        try:
+            self._login_limiter.try_acquire("login")
+        except BucketFullException as ex:
+            _LOGGER.info(ex.meta_info)
+            raise RoborockRateLimit("Reached maximum requests for login. Please try again later.") from ex
         base_url = await self._get_base_url()
         header_clientid = self._get_header_client_id()
         code_request = PreparedRequest(base_url, self.session, {"header_clientid": header_clientid})
@@ -191,6 +182,7 @@ class RoborockApiClient:
             raise RoborockException("Failed to get a response from send email code")
         response_code = code_response.get("code")
         if response_code != 200:
+            _LOGGER.info("Request code failed for %s with the following context: %s", self._username, code_response)
             if response_code == 2008:
                 raise RoborockAccountDoesNotExist("Account does not exist - check your login and try again.")
             elif response_code == 9002:
@@ -199,6 +191,11 @@ class RoborockApiClient:
                 raise RoborockException(f"{code_response.get('msg')} - response code: {code_response.get('code')}")
 
     async def pass_login(self, password: str) -> UserData:
+        try:
+            self._login_limiter.try_acquire("login")
+        except BucketFullException as ex:
+            _LOGGER.info(ex.meta_info)
+            raise RoborockRateLimit("Reached maximum requests for login. Please try again later.") from ex
         base_url = await self._get_base_url()
         header_clientid = self._get_header_client_id()
 
@@ -215,6 +212,7 @@ class RoborockApiClient:
         if login_response is None:
             raise RoborockException("Login response is none")
         if login_response.get("code") != 200:
+            _LOGGER.info("Login failed for %s with the following context: %s", self._username, login_response)
             raise RoborockException(f"{login_response.get('msg')} - response code: {login_response.get('code')}")
         user_data = login_response.get("data")
         if not isinstance(user_data, dict):
@@ -254,6 +252,7 @@ class RoborockApiClient:
             raise RoborockException("Login request response is None")
         response_code = login_response.get("code")
         if response_code != 200:
+            _LOGGER.info("Login failed for %s with the following context: %s", self._username, login_response)
             if response_code == 2018:
                 raise RoborockInvalidCode("Invalid code - check your code and try again.")
             if response_code == 3009:
@@ -280,6 +279,7 @@ class RoborockApiClient:
         if home_id_response is None:
             raise RoborockException("home_id_response is None")
         if home_id_response.get("code") != 200:
+            _LOGGER.info("Get Home Id failed with the following context: %s", home_id_response)
             if home_id_response.get("code") == 2010:
                 raise RoborockInvalidCredentials(
                     f"Invalid credentials ({home_id_response.get('msg')}) - check your login and try again."
@@ -289,6 +289,11 @@ class RoborockApiClient:
         return home_id_response["data"]["rrHomeId"]
 
     async def get_home_data(self, user_data: UserData) -> HomeData:
+        try:
+            self._home_data_limiter.try_acquire("home_data")
+        except BucketFullException as ex:
+            _LOGGER.info(ex.meta_info)
+            raise RoborockRateLimit("Reached maximum requests for home data. Please try again later.") from ex
         rriot = user_data.rriot
         if rriot is None:
             raise RoborockException("rriot is none")
@@ -299,7 +304,7 @@ class RoborockApiClient:
             rriot.r.a,
             self.session,
             {
-                "Authorization": self._get_hawk_authentication(rriot, f"/user/homes/{str(home_id)}"),
+                "Authorization": _get_hawk_authentication(rriot, f"/user/homes/{str(home_id)}"),
             },
         )
         home_response = await home_request.request("get", "/user/homes/" + str(home_id))
@@ -313,6 +318,11 @@ class RoborockApiClient:
 
     async def get_home_data_v2(self, user_data: UserData) -> HomeData:
         """This is the same as get_home_data, but uses a different endpoint and includes non-robotic vacuums."""
+        try:
+            self._home_data_limiter.try_acquire("home_data")
+        except BucketFullException as ex:
+            _LOGGER.info(ex.meta_info)
+            raise RoborockRateLimit("Reached maximum requests for home data. Please try again later.") from ex
         rriot = user_data.rriot
         if rriot is None:
             raise RoborockException("rriot is none")
@@ -323,7 +333,7 @@ class RoborockApiClient:
             rriot.r.a,
             self.session,
             {
-                "Authorization": self._get_hawk_authentication(rriot, "/v2/user/homes/" + str(home_id)),
+                "Authorization": _get_hawk_authentication(rriot, "/v2/user/homes/" + str(home_id)),
             },
         )
         home_response = await home_request.request("get", "/v2/user/homes/" + str(home_id))
@@ -337,17 +347,23 @@ class RoborockApiClient:
 
     async def get_home_data_v3(self, user_data: UserData) -> HomeData:
         """This is the same as get_home_data, but uses a different endpoint and includes non-robotic vacuums."""
+        try:
+            self._home_data_limiter.try_acquire("home_data")
+        except BucketFullException as ex:
+            _LOGGER.info(ex.meta_info)
+            raise RoborockRateLimit("Reached maximum requests for home data. Please try again later.") from ex
         rriot = user_data.rriot
         home_id = await self._get_home_id(user_data)
         if rriot.r.a is None:
             raise RoborockException("Missing field 'a' in rriot reference")
         home_request = PreparedRequest(
             rriot.r.a,
+            self.session,
             {
-                "Authorization": self._get_hawk_authentication(rriot, "/v3/user/homes/" + home_id),
+                "Authorization": _get_hawk_authentication(rriot, "/v3/user/homes/" + str(home_id)),
             },
         )
-        home_response = await home_request.request("get", "/v3/user/homes/" + home_id)
+        home_response = await home_request.request("get", "/v3/user/homes/" + str(home_id))
         if not home_response.get("success"):
             raise RoborockException(home_response)
         home_data = home_response.get("result")
@@ -367,7 +383,7 @@ class RoborockApiClient:
             rriot.r.a,
             self.session,
             {
-                "Authorization": self._get_hawk_authentication(rriot, "/v2/user/homes/" + str(home_id)),
+                "Authorization": _get_hawk_authentication(rriot, "/v2/user/homes/" + str(home_id)),
             },
         )
         room_response = await room_request.request("get", f"/user/homes/{str(home_id)}/rooms" + str(home_id))
@@ -392,7 +408,7 @@ class RoborockApiClient:
             rriot.r.a,
             self.session,
             {
-                "Authorization": self._get_hawk_authentication(rriot, f"/user/scene/device/{str(device_id)}"),
+                "Authorization": _get_hawk_authentication(rriot, f"/user/scene/device/{str(device_id)}"),
             },
         )
         scenes_response = await scenes_request.request("get", f"/user/scene/device/{str(device_id)}")
@@ -414,7 +430,7 @@ class RoborockApiClient:
             rriot.r.a,
             self.session,
             {
-                "Authorization": self._get_hawk_authentication(rriot, f"/user/scene/{str(scene_id)}/execute"),
+                "Authorization": _get_hawk_authentication(rriot, f"/user/scene/{str(scene_id)}/execute"),
             },
         )
         execute_scene_response = await execute_scene_request.request("POST", f"/user/scene/{str(scene_id)}/execute")
@@ -497,3 +513,36 @@ class PreparedRequest:
         finally:
             if close_session:
                 await session.close()
+
+
+def _process_extra_hawk_values(values: dict | None) -> str:
+    if values is None:
+        return ""
+    else:
+        sorted_keys = sorted(values.keys())
+        result = []
+        for key in sorted_keys:
+            value = values.get(key)
+            result.append(f"{key}={value}")
+        return hashlib.md5("&".join(result).encode()).hexdigest()
+
+
+def _get_hawk_authentication(rriot: RRiot, url: str, formdata: dict | None = None, params: dict | None = None) -> str:
+    timestamp = math.floor(time.time())
+    nonce = secrets.token_urlsafe(6)
+    formdata_str = _process_extra_hawk_values(formdata)
+    params_str = _process_extra_hawk_values(params)
+
+    prestr = ":".join(
+        [
+            rriot.u,
+            rriot.s,
+            nonce,
+            str(timestamp),
+            hashlib.md5(url.encode()).hexdigest(),
+            params_str,
+            formdata_str,
+        ]
+    )
+    mac = base64.b64encode(hmac.new(rriot.h.encode(), prestr.encode(), hashlib.sha256).digest()).decode()
+    return f'Hawk id="{rriot.u}",s="{rriot.s}",ts="{timestamp}",nonce="{nonce}",mac="{mac}"'
